"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.swaggerJSON = void 0;
const swaggerTemplate_1 = __importDefault(require("./swaggerTemplate"));
const utils_1 = require("./utils");
const ramda_1 = require("ramda");
/**
 * build swagger json from apiObjects
 */
const swaggerJSON = (options = {}, apiObjects) => {
    const { title, description, version, prefix = '', swaggerOptions = {} } = options;
    const swaggerJSON = swaggerTemplate_1.default(title, description, version, swaggerOptions);
    const paths = {};
    Object.keys(apiObjects).forEach((key) => {
        const value = apiObjects[key];
        if (!Object.keys(value).includes('request')) {
            return;
        }
        const { method } = value.request;
        let { path } = value.request;
        path = utils_1.getPath(prefix, value.prefix ? `${value.prefix}${path}` : path); // 根据前缀补全path
        const summary = value.summary || '';
        const description = value.description || summary;
        const responses = value.responses || {
            200: { description: 'success' }
        };
        const { query = [], header = [], path: pathParams = [], order, tags, security, deprecated, requestBody, } = value;
        pathParams.forEach((o) => o.required = true); // path params should not be optional
        const parameters = [...pathParams, ...query, ...header];
        parameters.forEach((o) => o.schema = o.type && !o.schema ? ramda_1.clone(o) : o.schema); // compatiable for swagger v2
        // init path object first
        if (!paths[path]) {
            paths[path] = {};
        }
        paths[path][method] = {
            summary,
            description,
            parameters,
            responses,
            tags,
            security,
            deprecated,
            requestBody,
        };
        if (!paths[path]._order) {
            paths[path]._order = order;
        }
    });
    swaggerJSON.paths = utils_1.sortObject(paths, (path, length) => path._order || length, (path) => {
        const { _order } = path, restOfPathData = __rest(path, ["_order"]);
        return restOfPathData;
    });
    return swaggerJSON;
};
exports.swaggerJSON = swaggerJSON;
exports.default = swaggerJSON;
